from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import stream_sra_round

from decimal import Decimal, ROUND_HALF_UP, ROUND_HALF_EVEN


expected_verilog = """
module test
(

);

  reg CLK;
  reg RST;
  reg signed [32-1:0] xdata;
  reg signed [32-1:0] ydata;
  wire signed [32-1:0] zdata;

  main
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .xdata(xdata),
    .ydata(ydata),
    .zdata(zdata)
  );

  reg reset_done;
  reg end_of_sim;

  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    reset_done = 0;
    xdata = 0;
    ydata = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #1000;
    reset_done = 1;
    @(posedge CLK);
    #1;
    #10;
    @(posedge end_of_sim);
    #100;
    $finish;
  end

  reg [32-1:0] send_fsm;
  localparam send_fsm_init = 0;
  reg [32-1:0] send_count;
  reg [32-1:0] recv_fsm;
  localparam recv_fsm_init = 0;
  reg [32-1:0] recv_count;
  localparam send_fsm_1 = 1;
  localparam send_fsm_2 = 2;
  localparam send_fsm_3 = 3;
  localparam send_fsm_4 = 4;
  localparam send_fsm_5 = 5;
  localparam send_fsm_6 = 6;
  localparam send_fsm_7 = 7;
  localparam send_fsm_8 = 8;
  localparam send_fsm_9 = 9;
  localparam send_fsm_10 = 10;
  localparam send_fsm_11 = 11;
  localparam send_fsm_12 = 12;
  localparam send_fsm_13 = 13;
  localparam send_fsm_14 = 14;
  localparam send_fsm_15 = 15;
  localparam send_fsm_16 = 16;
  localparam send_fsm_17 = 17;
  localparam send_fsm_18 = 18;
  localparam send_fsm_19 = 19;
  localparam send_fsm_20 = 20;
  localparam send_fsm_21 = 21;
  localparam send_fsm_22 = 22;
  localparam send_fsm_23 = 23;
  localparam send_fsm_24 = 24;
  localparam send_fsm_25 = 25;
  localparam send_fsm_26 = 26;
  localparam send_fsm_27 = 27;
  localparam send_fsm_28 = 28;
  localparam send_fsm_29 = 29;
  localparam send_fsm_30 = 30;
  localparam send_fsm_31 = 31;
  localparam send_fsm_32 = 32;
  localparam send_fsm_33 = 33;
  localparam send_fsm_34 = 34;
  localparam send_fsm_35 = 35;
  localparam send_fsm_36 = 36;
  localparam send_fsm_37 = 37;
  localparam send_fsm_38 = 38;
  localparam send_fsm_39 = 39;
  localparam send_fsm_40 = 40;
  localparam send_fsm_41 = 41;
  localparam send_fsm_42 = 42;
  localparam send_fsm_43 = 43;
  localparam send_fsm_44 = 44;
  localparam send_fsm_45 = 45;
  localparam send_fsm_46 = 46;
  localparam send_fsm_47 = 47;
  localparam send_fsm_48 = 48;
  localparam send_fsm_49 = 49;
  localparam send_fsm_50 = 50;
  localparam send_fsm_51 = 51;
  localparam send_fsm_52 = 52;
  localparam send_fsm_53 = 53;
  localparam send_fsm_54 = 54;
  localparam send_fsm_55 = 55;
  localparam send_fsm_56 = 56;
  localparam send_fsm_57 = 57;
  localparam send_fsm_58 = 58;
  localparam send_fsm_59 = 59;
  localparam send_fsm_60 = 60;
  localparam send_fsm_61 = 61;
  localparam send_fsm_62 = 62;
  localparam send_fsm_63 = 63;
  localparam send_fsm_64 = 64;
  localparam send_fsm_65 = 65;
  localparam send_fsm_66 = 66;
  localparam send_fsm_67 = 67;
  localparam send_fsm_68 = 68;
  localparam send_fsm_69 = 69;
  localparam send_fsm_70 = 70;
  localparam send_fsm_71 = 71;
  localparam send_fsm_72 = 72;
  localparam send_fsm_73 = 73;
  localparam send_fsm_74 = 74;
  localparam send_fsm_75 = 75;
  localparam send_fsm_76 = 76;
  localparam send_fsm_77 = 77;
  localparam send_fsm_78 = 78;
  localparam send_fsm_79 = 79;
  localparam send_fsm_80 = 80;
  localparam send_fsm_81 = 81;

  always @(posedge CLK) begin
    if(RST) begin
      send_fsm <= send_fsm_init;
      send_count <= 0;
    end else begin
      case(send_fsm)
        send_fsm_init: begin
          if(reset_done) begin
            send_fsm <= send_fsm_1;
          end 
        end
        send_fsm_1: begin
          xdata <= -2147483648;
          ydata <= 0;
          send_count <= 0;
          send_fsm <= send_fsm_2;
        end
        send_fsm_2: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_3;
          end 
        end
        send_fsm_3: begin
          xdata <= -10;
          ydata <= 0;
          send_count <= 0;
          send_fsm <= send_fsm_4;
        end
        send_fsm_4: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_5;
          end 
        end
        send_fsm_5: begin
          xdata <= 1;
          ydata <= 0;
          send_count <= 0;
          send_fsm <= send_fsm_6;
        end
        send_fsm_6: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_7;
          end 
        end
        send_fsm_7: begin
          xdata <= 2147483637;
          ydata <= 0;
          send_count <= 0;
          send_fsm <= send_fsm_8;
        end
        send_fsm_8: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_9;
          end 
        end
        send_fsm_9: begin
          xdata <= -2147483648;
          ydata <= 1;
          send_count <= 0;
          send_fsm <= send_fsm_10;
        end
        send_fsm_10: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_11;
          end 
        end
        send_fsm_11: begin
          xdata <= -10;
          ydata <= 1;
          send_count <= 0;
          send_fsm <= send_fsm_12;
        end
        send_fsm_12: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_13;
          end 
        end
        send_fsm_13: begin
          xdata <= 1;
          ydata <= 1;
          send_count <= 0;
          send_fsm <= send_fsm_14;
        end
        send_fsm_14: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_15;
          end 
        end
        send_fsm_15: begin
          xdata <= 2147483637;
          ydata <= 1;
          send_count <= 0;
          send_fsm <= send_fsm_16;
        end
        send_fsm_16: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_17;
          end 
        end
        send_fsm_17: begin
          xdata <= -2147483648;
          ydata <= 2;
          send_count <= 0;
          send_fsm <= send_fsm_18;
        end
        send_fsm_18: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_19;
          end 
        end
        send_fsm_19: begin
          xdata <= -10;
          ydata <= 2;
          send_count <= 0;
          send_fsm <= send_fsm_20;
        end
        send_fsm_20: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_21;
          end 
        end
        send_fsm_21: begin
          xdata <= 1;
          ydata <= 2;
          send_count <= 0;
          send_fsm <= send_fsm_22;
        end
        send_fsm_22: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_23;
          end 
        end
        send_fsm_23: begin
          xdata <= 2147483637;
          ydata <= 2;
          send_count <= 0;
          send_fsm <= send_fsm_24;
        end
        send_fsm_24: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_25;
          end 
        end
        send_fsm_25: begin
          xdata <= -2147483648;
          ydata <= 3;
          send_count <= 0;
          send_fsm <= send_fsm_26;
        end
        send_fsm_26: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_27;
          end 
        end
        send_fsm_27: begin
          xdata <= -10;
          ydata <= 3;
          send_count <= 0;
          send_fsm <= send_fsm_28;
        end
        send_fsm_28: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_29;
          end 
        end
        send_fsm_29: begin
          xdata <= 1;
          ydata <= 3;
          send_count <= 0;
          send_fsm <= send_fsm_30;
        end
        send_fsm_30: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_31;
          end 
        end
        send_fsm_31: begin
          xdata <= 2147483637;
          ydata <= 3;
          send_count <= 0;
          send_fsm <= send_fsm_32;
        end
        send_fsm_32: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_33;
          end 
        end
        send_fsm_33: begin
          xdata <= -2147483648;
          ydata <= 15;
          send_count <= 0;
          send_fsm <= send_fsm_34;
        end
        send_fsm_34: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_35;
          end 
        end
        send_fsm_35: begin
          xdata <= -10;
          ydata <= 15;
          send_count <= 0;
          send_fsm <= send_fsm_36;
        end
        send_fsm_36: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_37;
          end 
        end
        send_fsm_37: begin
          xdata <= 1;
          ydata <= 15;
          send_count <= 0;
          send_fsm <= send_fsm_38;
        end
        send_fsm_38: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_39;
          end 
        end
        send_fsm_39: begin
          xdata <= 2147483637;
          ydata <= 15;
          send_count <= 0;
          send_fsm <= send_fsm_40;
        end
        send_fsm_40: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_41;
          end 
        end
        send_fsm_41: begin
          xdata <= -2147483648;
          ydata <= 16;
          send_count <= 0;
          send_fsm <= send_fsm_42;
        end
        send_fsm_42: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_43;
          end 
        end
        send_fsm_43: begin
          xdata <= -10;
          ydata <= 16;
          send_count <= 0;
          send_fsm <= send_fsm_44;
        end
        send_fsm_44: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_45;
          end 
        end
        send_fsm_45: begin
          xdata <= 1;
          ydata <= 16;
          send_count <= 0;
          send_fsm <= send_fsm_46;
        end
        send_fsm_46: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_47;
          end 
        end
        send_fsm_47: begin
          xdata <= 2147483637;
          ydata <= 16;
          send_count <= 0;
          send_fsm <= send_fsm_48;
        end
        send_fsm_48: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_49;
          end 
        end
        send_fsm_49: begin
          xdata <= -2147483648;
          ydata <= 17;
          send_count <= 0;
          send_fsm <= send_fsm_50;
        end
        send_fsm_50: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_51;
          end 
        end
        send_fsm_51: begin
          xdata <= -10;
          ydata <= 17;
          send_count <= 0;
          send_fsm <= send_fsm_52;
        end
        send_fsm_52: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_53;
          end 
        end
        send_fsm_53: begin
          xdata <= 1;
          ydata <= 17;
          send_count <= 0;
          send_fsm <= send_fsm_54;
        end
        send_fsm_54: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_55;
          end 
        end
        send_fsm_55: begin
          xdata <= 2147483637;
          ydata <= 17;
          send_count <= 0;
          send_fsm <= send_fsm_56;
        end
        send_fsm_56: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_57;
          end 
        end
        send_fsm_57: begin
          xdata <= -2147483648;
          ydata <= 30;
          send_count <= 0;
          send_fsm <= send_fsm_58;
        end
        send_fsm_58: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_59;
          end 
        end
        send_fsm_59: begin
          xdata <= -10;
          ydata <= 30;
          send_count <= 0;
          send_fsm <= send_fsm_60;
        end
        send_fsm_60: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_61;
          end 
        end
        send_fsm_61: begin
          xdata <= 1;
          ydata <= 30;
          send_count <= 0;
          send_fsm <= send_fsm_62;
        end
        send_fsm_62: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_63;
          end 
        end
        send_fsm_63: begin
          xdata <= 2147483637;
          ydata <= 30;
          send_count <= 0;
          send_fsm <= send_fsm_64;
        end
        send_fsm_64: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_65;
          end 
        end
        send_fsm_65: begin
          xdata <= -2147483648;
          ydata <= 31;
          send_count <= 0;
          send_fsm <= send_fsm_66;
        end
        send_fsm_66: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_67;
          end 
        end
        send_fsm_67: begin
          xdata <= -10;
          ydata <= 31;
          send_count <= 0;
          send_fsm <= send_fsm_68;
        end
        send_fsm_68: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_69;
          end 
        end
        send_fsm_69: begin
          xdata <= 1;
          ydata <= 31;
          send_count <= 0;
          send_fsm <= send_fsm_70;
        end
        send_fsm_70: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_71;
          end 
        end
        send_fsm_71: begin
          xdata <= 2147483637;
          ydata <= 31;
          send_count <= 0;
          send_fsm <= send_fsm_72;
        end
        send_fsm_72: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_73;
          end 
        end
        send_fsm_73: begin
          xdata <= -2147483648;
          ydata <= 32;
          send_count <= 0;
          send_fsm <= send_fsm_74;
        end
        send_fsm_74: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_75;
          end 
        end
        send_fsm_75: begin
          xdata <= -10;
          ydata <= 32;
          send_count <= 0;
          send_fsm <= send_fsm_76;
        end
        send_fsm_76: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_77;
          end 
        end
        send_fsm_77: begin
          xdata <= 1;
          ydata <= 32;
          send_count <= 0;
          send_fsm <= send_fsm_78;
        end
        send_fsm_78: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_79;
          end 
        end
        send_fsm_79: begin
          xdata <= 2147483637;
          ydata <= 32;
          send_count <= 0;
          send_fsm <= send_fsm_80;
        end
        send_fsm_80: begin
          xdata <= xdata + 1;
          send_count <= send_count + 1;
          $display("xdata=%d", xdata);
          $display("ydata=%d", ydata);
          if(send_count == 9) begin
            send_fsm <= send_fsm_81;
          end 
        end
      endcase
    end
  end

  localparam recv_fsm_1 = 1;
  localparam recv_fsm_2 = 2;
  localparam recv_fsm_3 = 3;
  localparam recv_fsm_4 = 4;
  localparam recv_fsm_5 = 5;
  localparam recv_fsm_6 = 6;
  localparam recv_fsm_7 = 7;
  localparam recv_fsm_8 = 8;
  localparam recv_fsm_9 = 9;
  localparam recv_fsm_10 = 10;
  localparam recv_fsm_11 = 11;
  localparam recv_fsm_12 = 12;
  localparam recv_fsm_13 = 13;
  localparam recv_fsm_14 = 14;
  localparam recv_fsm_15 = 15;
  localparam recv_fsm_16 = 16;
  localparam recv_fsm_17 = 17;
  localparam recv_fsm_18 = 18;
  localparam recv_fsm_19 = 19;
  localparam recv_fsm_20 = 20;
  localparam recv_fsm_21 = 21;
  localparam recv_fsm_22 = 22;
  localparam recv_fsm_23 = 23;
  localparam recv_fsm_24 = 24;
  localparam recv_fsm_25 = 25;
  localparam recv_fsm_26 = 26;
  localparam recv_fsm_27 = 27;
  localparam recv_fsm_28 = 28;
  localparam recv_fsm_29 = 29;
  localparam recv_fsm_30 = 30;
  localparam recv_fsm_31 = 31;
  localparam recv_fsm_32 = 32;
  localparam recv_fsm_33 = 33;
  localparam recv_fsm_34 = 34;
  localparam recv_fsm_35 = 35;
  localparam recv_fsm_36 = 36;
  localparam recv_fsm_37 = 37;
  localparam recv_fsm_38 = 38;
  localparam recv_fsm_39 = 39;
  localparam recv_fsm_40 = 40;
  localparam recv_fsm_41 = 41;
  localparam recv_fsm_42 = 42;
  localparam recv_fsm_43 = 43;
  localparam recv_fsm_44 = 44;
  localparam recv_fsm_45 = 45;
  localparam recv_fsm_46 = 46;
  localparam recv_fsm_47 = 47;
  localparam recv_fsm_48 = 48;
  localparam recv_fsm_49 = 49;
  localparam recv_fsm_50 = 50;
  localparam recv_fsm_51 = 51;
  localparam recv_fsm_52 = 52;
  localparam recv_fsm_53 = 53;
  localparam recv_fsm_54 = 54;
  localparam recv_fsm_55 = 55;
  localparam recv_fsm_56 = 56;
  localparam recv_fsm_57 = 57;
  localparam recv_fsm_58 = 58;
  localparam recv_fsm_59 = 59;
  localparam recv_fsm_60 = 60;
  localparam recv_fsm_61 = 61;
  localparam recv_fsm_62 = 62;
  localparam recv_fsm_63 = 63;
  localparam recv_fsm_64 = 64;
  localparam recv_fsm_65 = 65;
  localparam recv_fsm_66 = 66;
  localparam recv_fsm_67 = 67;
  localparam recv_fsm_68 = 68;
  localparam recv_fsm_69 = 69;
  localparam recv_fsm_70 = 70;
  localparam recv_fsm_71 = 71;
  localparam recv_fsm_72 = 72;
  localparam recv_fsm_73 = 73;
  localparam recv_fsm_74 = 74;
  localparam recv_fsm_75 = 75;
  localparam recv_fsm_76 = 76;
  localparam recv_fsm_77 = 77;
  localparam recv_fsm_78 = 78;
  localparam recv_fsm_79 = 79;
  localparam recv_fsm_80 = 80;
  localparam recv_fsm_81 = 81;
  localparam recv_fsm_82 = 82;

  always @(posedge CLK) begin
    if(RST) begin
      recv_fsm <= recv_fsm_init;
      recv_count <= 0;
      end_of_sim <= 0;
    end else begin
      case(recv_fsm)
        recv_fsm_init: begin
          if(reset_done) begin
            recv_fsm <= recv_fsm_1;
          end 
        end
        recv_fsm_1: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_2;
        end
        recv_fsm_2: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_3;
        end
        recv_fsm_3: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_4;
          end 
        end
        recv_fsm_4: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_5;
        end
        recv_fsm_5: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_6;
          end 
        end
        recv_fsm_6: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_7;
        end
        recv_fsm_7: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_8;
          end 
        end
        recv_fsm_8: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_9;
        end
        recv_fsm_9: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_10;
          end 
        end
        recv_fsm_10: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_11;
        end
        recv_fsm_11: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_12;
          end 
        end
        recv_fsm_12: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_13;
        end
        recv_fsm_13: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_14;
          end 
        end
        recv_fsm_14: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_15;
        end
        recv_fsm_15: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_16;
          end 
        end
        recv_fsm_16: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_17;
        end
        recv_fsm_17: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_18;
          end 
        end
        recv_fsm_18: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_19;
        end
        recv_fsm_19: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_20;
          end 
        end
        recv_fsm_20: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_21;
        end
        recv_fsm_21: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_22;
          end 
        end
        recv_fsm_22: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_23;
        end
        recv_fsm_23: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_24;
          end 
        end
        recv_fsm_24: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_25;
        end
        recv_fsm_25: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_26;
          end 
        end
        recv_fsm_26: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_27;
        end
        recv_fsm_27: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_28;
          end 
        end
        recv_fsm_28: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_29;
        end
        recv_fsm_29: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_30;
          end 
        end
        recv_fsm_30: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_31;
        end
        recv_fsm_31: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_32;
          end 
        end
        recv_fsm_32: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_33;
        end
        recv_fsm_33: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_34;
          end 
        end
        recv_fsm_34: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_35;
        end
        recv_fsm_35: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_36;
          end 
        end
        recv_fsm_36: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_37;
        end
        recv_fsm_37: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_38;
          end 
        end
        recv_fsm_38: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_39;
        end
        recv_fsm_39: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_40;
          end 
        end
        recv_fsm_40: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_41;
        end
        recv_fsm_41: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_42;
          end 
        end
        recv_fsm_42: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_43;
        end
        recv_fsm_43: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_44;
          end 
        end
        recv_fsm_44: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_45;
        end
        recv_fsm_45: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_46;
          end 
        end
        recv_fsm_46: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_47;
        end
        recv_fsm_47: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_48;
          end 
        end
        recv_fsm_48: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_49;
        end
        recv_fsm_49: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_50;
          end 
        end
        recv_fsm_50: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_51;
        end
        recv_fsm_51: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_52;
          end 
        end
        recv_fsm_52: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_53;
        end
        recv_fsm_53: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_54;
          end 
        end
        recv_fsm_54: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_55;
        end
        recv_fsm_55: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_56;
          end 
        end
        recv_fsm_56: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_57;
        end
        recv_fsm_57: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_58;
          end 
        end
        recv_fsm_58: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_59;
        end
        recv_fsm_59: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_60;
          end 
        end
        recv_fsm_60: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_61;
        end
        recv_fsm_61: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_62;
          end 
        end
        recv_fsm_62: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_63;
        end
        recv_fsm_63: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_64;
          end 
        end
        recv_fsm_64: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_65;
        end
        recv_fsm_65: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_66;
          end 
        end
        recv_fsm_66: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_67;
        end
        recv_fsm_67: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_68;
          end 
        end
        recv_fsm_68: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_69;
        end
        recv_fsm_69: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_70;
          end 
        end
        recv_fsm_70: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_71;
        end
        recv_fsm_71: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_72;
          end 
        end
        recv_fsm_72: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_73;
        end
        recv_fsm_73: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_74;
          end 
        end
        recv_fsm_74: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_75;
        end
        recv_fsm_75: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_76;
          end 
        end
        recv_fsm_76: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_77;
        end
        recv_fsm_77: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_78;
          end 
        end
        recv_fsm_78: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_79;
        end
        recv_fsm_79: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_80;
          end 
        end
        recv_fsm_80: begin
          recv_count <= 0;
          recv_fsm <= recv_fsm_81;
        end
        recv_fsm_81: begin
          $display("zdata=%d", zdata);
          recv_count <= recv_count + 1;
          if(recv_count == 9) begin
            recv_fsm <= recv_fsm_82;
          end 
        end
        recv_fsm_82: begin
          end_of_sim <= 1;
        end
      endcase
    end
  end


endmodule



module main
(
  input CLK,
  input RST,
  input signed [32-1:0] xdata,
  input signed [32-1:0] ydata,
  output signed [32-1:0] zdata
);

  wire [1-1:0] _pointer_data_2;
  assign _pointer_data_2 = xdata[6'sd31];
  wire [5-1:0] _slice_data_6;
  assign _slice_data_6 = ydata[4'd4:1'd0];
  wire [5-1:0] _minus_data_7;
  assign _minus_data_7 = _slice_data_6 - 2'sd1;
  wire signed [34-1:0] _sll_data_10;
  assign _sll_data_10 = 2'sd1 << _minus_data_7;
  wire signed [2-1:0] _cond_data_23;
  assign _cond_data_23 = (_pointer_data_2)? -2'sd1 : 1'sd0;
  wire signed [33-1:0] _plus_data_24;
  assign _plus_data_24 = xdata + _sll_data_10;
  wire signed [33-1:0] _plus_data_25;
  assign _plus_data_25 = _plus_data_24 + _cond_data_23;
  wire signed [32-1:0] _sra_data_26;
  assign _sra_data_26 = _plus_data_25 >>> ydata;
  wire [1-1:0] _eq_data_28;
  assign _eq_data_28 = ydata == 1'sd0;
  reg signed [32-1:0] _cond_data_29;
  assign zdata = _cond_data_29;

  always @(posedge CLK) begin
    if(RST) begin
      _cond_data_29 <= 0;
    end else begin
      _cond_data_29 <= (_eq_data_28)? xdata : _sra_data_26;
    end
  end


endmodule

"""


def test():
    veriloggen.reset()
    test_module = stream_sra_round.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)

    # run simulator (Icarus Verilog)
    sim = veriloggen.simulation.Simulator(test_module)
    rslt = sim.run()  # display=False
    #rslt = sim.run(display=True)
    print(rslt)

    vx = list(map(lambda x: int(str.split(x, "=")[1]),
                  filter(lambda x: "xdata" in x, str.split(rslt, "\n"))))
    vy = list(map(lambda x: int(str.split(x, "=")[1]),
                  filter(lambda x: "ydata" in x, str.split(rslt, "\n"))))
    vz = list(map(lambda x: int(str.split(x, "=")[1]),
                  filter(lambda x: "zdata" in x, str.split(rslt, "\n"))))
    ez = list(map(lambda x, y:
                  int(Decimal(str(x / (2.0**y))).quantize(Decimal('0'),
                                                          rounding=ROUND_HALF_UP)), vx, vy))

    assert(all(map(lambda x, y: x == y, vz, ez)))



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))))

from veriloggen import *
import veriloggen.types.axi as axi
from veriloggen.thread import RAM
import veriloggen.dataflow as dataflow


def mkMain():
    m = Module('main')
    clk = m.Input('CLK')
    rst = m.Input('RST')

    myaxi = axi.AxiMaster(m, 'myaxi', clk, rst)
    myaxi.disable_write()

    myram = RAM(m, 'myram', clk, rst, numports=1)

    df = dataflow.DataflowManager(m, clk, rst)
    fsm = FSM(m, 'fsm', clk, rst)

    # AXI read request
    araddr = 1024
    arlen = 64
    ack, axi_counter = myaxi.read_request_counter(araddr, arlen, cond=fsm)
    fsm.If(ack).goto_next()

    # AXI read dataflow (AXI -> Dataflow)
    axi_data, axi_last, done = myaxi.read_dataflow()
    sum = df.ReduceAdd(axi_data, reset=axi_last.prev(1))

    # RAM write dataflow (Dataflow -> RAM)
    wport = 0
    waddr = 0
    wlen = arlen
    done = myram.write_dataflow(wport, waddr, sum, wlen, cond=fsm)
    fsm.goto_next()
    fsm.If(done).goto_next()

    # verify
    # read dataflow (RAM -> Dataflow)
    rport = 0
    raddr = 0
    rlen = arlen
    rdata, rlast, done = myram.read_dataflow(rport, raddr, rlen, cond=fsm)
    fsm.goto_next()
    fsm.If(done).goto_next()

    rdata_data, rdata_valid = rdata.read()
    rlast_data, rlast_valid = rlast.read()

    sum = m.Reg('sum', 32, initval=0)
    expected_sum = 0
    for i in range(arlen):
        expected_sum += (araddr + araddr + i) * (i + 1) // 2

    seq = Seq(m, 'seq', clk, rst)

    seq.If(rdata_valid)(
        sum.add(rdata_data)
    )
    seq.Then().If(rlast_data == 1).Delay(1)(
        Systask('display', 'sum=%d expected_sum=%d', sum, expected_sum),
        If(NotEql(sum, expected_sum))(Display('# verify: FAILED')).Else(Display('# verify: PASSED'))
    )

    return m


def mkTest(memimg_name=None):
    m = Module('test')

    # target instance
    main = mkMain()

    # copy paras and ports
    params = m.copy_params(main)
    ports = m.copy_sim_ports(main)

    clk = ports['CLK']
    rst = ports['RST']

    # awready (no stall)
    awready = ports['myaxi_awready']
    _awready = m.TmpWireLike(awready)
    _awready.assign(1)
    m.Always()(awready(_awready))

    # wready (nostall)
    wready = ports['myaxi_wready']
    _wready = m.TmpWireLike(wready)
    _wready.assign(1)
    m.Always()(wready(_wready))

    # arready (no stall)
    #arready = ports['myaxi_arready']
    #_arready = m.TmpWireLike(arready)
    # _arready.assign(0)
    #m.Always()( arready(_arready) )

    # arready, rvalid, rdata, rlast
    raddr_fsm = FSM(m, 'raddr', clk, rst)
    _arlen = m.Reg('_arlen', 32, initval=0)
    raddr_fsm(
        ports['myaxi_arready'](0),
        ports['myaxi_rdata'](-1),
        ports['myaxi_rvalid'](0),
        ports['myaxi_rlast'](0)
    )
    raddr_fsm.If(ports['myaxi_arvalid']).goto_next()

    raddr_fsm.If(ports['myaxi_arvalid'])(
        ports['myaxi_arready'](1),
        ports['myaxi_rdata'](ports['myaxi_araddr'] - 1)
    )
    raddr_fsm.goto_next()

    raddr_fsm(
        ports['myaxi_arready'](0),
        _arlen(ports['myaxi_arlen'])
    )
    raddr_fsm.goto_next()

    ack = Ors(ports['myaxi_rready'], Not(ports['myaxi_rvalid']))

    # nodelay
#    raddr_fsm.If(Ands(ack, Not(ports['myaxi_rlast'])))(
#        ports['myaxi_rdata'].inc(),
#        ports['myaxi_rvalid'](1),
#        ports['myaxi_rlast'](0),
#        _arlen.dec()
#    )
#    raddr_fsm.Then().If(_arlen == 0)(
#        ports['myaxi_rlast'](1),
#    )
#    raddr_fsm.Delay(1)(
#        ports['myaxi_rvalid'](0),
#        ports['myaxi_rlast'](0)
#    )
#    raddr_fsm.If(Ands(ports['myaxi_rvalid'], Not(ports['myaxi_rready'])))(
#        ports['myaxi_rvalid'](ports['myaxi_rvalid']),
#        ports['myaxi_rlast'](ports['myaxi_rlast']),
#    )
#    raddr_fsm.If(Ands(ports['myaxi_rvalid'], ports[
#                 'myaxi_rready'], ports['myaxi_rlast'])).goto_next()

    rdata_head = raddr_fsm.current
    raddr_fsm.If(Ands(ack, Not(ports['myaxi_rlast'])))(
        ports['myaxi_rdata'].inc(),
        ports['myaxi_rvalid'](1),
        ports['myaxi_rlast'](0),
        _arlen.dec()
    )
    raddr_fsm.Then().If(_arlen == 0)(
        ports['myaxi_rlast'](1)
    )
    raddr_fsm.Delay(1)(
        ports['myaxi_rvalid'](0),
        ports['myaxi_rlast'](0)
    )
    raddr_fsm.goto_next()
    raddr_fsm.If(Ands(ports['myaxi_rvalid'], Not(ports['myaxi_rready'])))(
        ports['myaxi_rvalid'](ports['myaxi_rvalid']),
        ports['myaxi_rlast'](ports['myaxi_rlast']),
    )
    raddr_fsm.If(Ands(ports['myaxi_rvalid'],
                      ports['myaxi_rready'])).goto_next()
    raddr_fsm.goto_next()
    raddr_fsm.goto_next()
    raddr_fsm.goto_next()
    raddr_fsm.If((_arlen + 1) & 0xff != 0).goto(rdata_head)
    raddr_fsm.If((_arlen + 1) & 0xff == 0).goto_next()

    raddr_fsm.goto_next()

    raddr_fsm.goto_init()

    raddr_fsm.make_always()

    uut = m.Instance(main, 'uut',
                     params=m.connect_params(main),
                     ports=m.connect_ports(main))

    # simulation.setup_waveform(m, uut, m.get_vars())
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)

    init.add(
        Delay(1000 * 100),
        Systask('finish'),
    )

    return m


def run(filename='tmp.v', simtype='iverilog', outputfile=None):

    if outputfile is None:
        outputfile = os.path.splitext(os.path.basename(__file__))[0] + '.out'

    memimg_name = 'memimg_' + outputfile

    test = mkTest(memimg_name=memimg_name)

    if filename is not None:
        test.to_verilog(filename)

    sim = simulation.Simulator(test, sim=simtype)
    rslt = sim.run(outputfile=outputfile)
    lines = rslt.splitlines()
    if simtype == 'verilator' and lines[-1].startswith('-'):
        rslt = '\n'.join(lines[:-1])
    return rslt


if __name__ == '__main__':
    rslt = run(filename='tmp.v')
    print(rslt)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))

from veriloggen import *
import veriloggen.thread as vthread


def mkLed():
    m = Module('blinkled')
    clk = m.Input('CLK')
    rst = m.Input('RST')

    datawidth = 32
    addrwidth = 10
    myram = vthread.RAM(m, 'myram', clk, rst, datawidth, addrwidth,
                        ram_style='(* ram_style = "block" *)')

    def blink(times):
        all_ok = True

        write_sum = 0
        for i in range(times):
            wdata = i
            myram.write(i, wdata)
            write_sum += wdata
            print('wdata = %d' % wdata)

        read_sum = 0
        for i in range(times):
            rdata = myram.read(i)
            read_sum += rdata
            print('rdata = %d' % rdata)
            if vthread.verilog.NotEql(rdata, i):
                all_ok = False

        print('read_sum = %d' % read_sum)

        if vthread.verilog.NotEql(read_sum, write_sum):
            all_ok = False

        if all_ok:
            print('# verify: PASSED')
        else:
            print('# verify: FAILED')

    th = vthread.Thread(m, 'th_blink', clk, rst, blink)
    fsm = th.start(10)

    return m


def mkTest(memimg_name=None):
    m = Module('test')

    # target instance
    led = mkLed()

    # copy paras and ports
    params = m.copy_params(led)
    ports = m.copy_sim_ports(led)

    clk = ports['CLK']
    rst = ports['RST']

    uut = m.Instance(led, 'uut',
                     params=m.connect_params(led),
                     ports=m.connect_ports(led))

    # simulation.setup_waveform(m, uut)
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)

    init.add(
        Delay(10000),
        Systask('finish'),
    )

    return m


def run(filename='tmp.v', simtype='iverilog', outputfile=None):

    if outputfile is None:
        outputfile = os.path.splitext(os.path.basename(__file__))[0] + '.out'

    memimg_name = 'memimg_' + outputfile

    test = mkTest(memimg_name=memimg_name)

    if filename is not None:
        test.to_verilog(filename)

    sim = simulation.Simulator(test, sim=simtype)
    rslt = sim.run(outputfile=outputfile)
    lines = rslt.splitlines()
    if simtype == 'verilator' and lines[-1].startswith('-'):
        rslt = '\n'.join(lines[:-1])
    return rslt


if __name__ == '__main__':
    rslt = run(filename='tmp.v')
    print(rslt)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os
import math

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))))

from veriloggen import *
import veriloggen.types.rom as rom


def mkMain(n=128, datawidth=32, numports=2):
    m = Module('main')

    clk = m.Input('CLK')
    rst = m.Input('RST')

    width = 4
    addr = m.Reg('addr', width, initval=0)
    values = [ i * i for i in range(2 ** width) ]

    myrom = rom.SyncROM(m, 'myrom', clk, addr, values, datawidth=8)

    seq = Seq(m, 'seq', clk, rst)

    seq(
        addr.inc(),
        Display('addr=%d rdata=%d', addr, myrom.rdata)
    )

    return m


def mkTest():
    m = Module('test')

    # target instance
    main = mkMain()

    # copy paras and ports
    params = m.copy_params(main)
    ports = m.copy_sim_ports(main)

    clk = ports['CLK']
    rst = ports['RST']

    uut = m.Instance(main, 'uut',
                     params=m.connect_params(main),
                     ports=m.connect_ports(main))

    simulation.setup_waveform(m, uut, m.get_vars())
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)

    init.add(
        Delay(1000 * 100),
        Systask('finish'),
    )

    return m

if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)

    sim = simulation.Simulator(test)
    rslt = sim.run()
    print(rslt)

    # sim.view_waveform()



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import logic

expected_verilog = """
module blinkled
(
  input a,
  input b,
  input c,
  output exp
);

  assign exp = a & b ^ ~c;

endmodule
"""

def test():
    veriloggen.reset()
    test_module = logic.mkLed()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)



from __future__ import absolute_import
from __future__ import print_function

import copy
import os
import sys
from collections import OrderedDict


class ScopeName(object):

    def __init__(self, namelist):
        self.namelist = namelist

    def __repr__(self):
        return '_'.join(self.namelist)


class ScopeFrameList(object):

    def __init__(self):
        self.scopeframes = []
        self.scopeframes.append(ScopeFrame(ScopeName(('_',))))
        self.current = self.scopeframes[0]
        self.globalframe = self.scopeframes[0]
        self.previousframes = OrderedDict()
        self.previousframes[self.current] = None
        self.nextframes = OrderedDict()
        self.label_prefix = 's'
        self.label_count = 0
        self.binds = OrderedDict()

    def getNamePrefix(self):
        return self.current.getNamePrefix()

    def getCurrent(self):
        return self.current

    def pushScopeFrame(self, name=None, ftype=None):
        if name is None:
            name = self.label_prefix + str(self.label_count)
            self.label_count += 1
        prefix = copy.deepcopy(self.current.name.namelist)
        framename = ScopeName(prefix + (name,))
        f = ScopeFrame(framename, ftype)
        self.scopeframes.append(f)
        self.previousframes[f] = self.current
        if self.current not in self.nextframes:
            self.nextframes[self.current] = []
        self.nextframes[self.current].append(f)
        self.current = f

    def popScopeFrame(self):
        self.current = self.previousframes[self.current]

    def addVariable(self, name, var):
        if self.current is None:
            return None
        targ = self.current
        while targ is not None:
            if targ.ftype == 'call':
                targ.addVariable(name, var)
                break
            targ = self.previousframes[targ]
        return None

    def addNonlocal(self, name):
        if self.current is None:
            return None
        targ = self.current
        while targ is not None:
            if targ.ftype == 'call':
                targ.addNonlocal(name)
                break
            targ = self.previousframes[targ]
        return None

    def addGlobal(self, name):
        if self.current is None:
            return None
        targ = self.current
        while targ is not None:
            if targ.ftype == 'call':
                targ.addGlobal(name)
                break
            targ = self.previousframes[targ]
        return None

    def addFunction(self, func):
        self.current.addFunction(func)

    def searchVariable(self, name, store=False):
        if self.current is None:
            return None
        targ = self.current
        is_global = False
        while targ is not None:
            ret = targ.searchVariable(name)
            if ret is not None:
                return ret
            if targ.ftype == 'call':
                ret = targ.searchNonlocal(name)
                if ret:
                    continue
                ret = targ.searchGlobal(name)
                if ret:
                    is_global = True
                break
            targ = self.previousframes[targ]
        if not store or is_global:
            ret = self.globalframe.searchVariable(name)
            return ret
        return None

    def searchFunction(self, name):
        if self.current is None:
            return None
        targ = self.current
        while targ is not None:
            ret = targ.searchFunction(name)
            if ret:
                return ret
            targ = self.previousframes[targ]
        ret = self.globalframe.searchFunction(name)
        return ret

    def addBind(self, state, dst, var, cond=None):
        if dst not in self.binds:
            self.binds[dst] = []
        self.binds[dst].append((state, var, cond))

    def getBinds(self):
        return self.binds

    def addBreak(self, count):
        self.current.addBreak(count)

    def addContinue(self, count):
        self.current.addContinue(count)

    def addReturn(self, count, value):
        self.current.addReturn(count, value)

    def setReturnVariable(self, var):
        if self.current is None:
            return
        targ = self.current
        while targ is not None:
            if targ.ftype == 'call':
                targ.setReturnVariable(var)
                return
            targ = self.previousframes[targ]

    def hasBreak(self):
        return self.current.hasBreak()

    def hasContinue(self):
        return self.current.hasContinue()

    def hasReturn(self):
        return self.current.hasReturn()

    def getUnresolvedBreak(self, p=None):
        ret = []
        ptr = self.current if p is None else p
        ret.extend(ptr.getBreak())
        if ptr not in self.nextframes:
            return tuple(ret)
        for f in self.nextframes[ptr]:
            ret.extend(self.getUnresolvedBreak(f))
        return tuple(ret)

    def getUnresolvedContinue(self, p=None):
        ret = []
        ptr = self.current if p is None else p
        ret.extend(ptr.getContinue())
        if ptr not in self.nextframes:
            return tuple(ret)
        for f in self.nextframes[ptr]:
            ret.extend(self.getUnresolvedContinue(f))
        return tuple(ret)

    def getUnresolvedReturn(self, p=None):
        ret = []
        ptr = self.current if p is None else p
        ret.extend(ptr.getReturn())
        if ptr not in self.nextframes:
            return tuple(ret)
        for f in self.nextframes[ptr]:
            ret.extend(self.getUnresolvedReturn(f))
        return tuple(ret)

    def getReturnVariable(self):
        if self.current is None:
            return None
        targ = self.current
        while targ is not None:
            if targ.ftype == 'call':
                return targ.getReturnVariable()
            targ = self.previousframes[targ]
        return None

    def clearBreak(self, p=None):
        ptr = self.current if p is None else p
        ptr.clearBreak()
        if ptr not in self.nextframes:
            return
        for f in self.nextframes[ptr]:
            self.clearBreak(f)

    def clearContinue(self, p=None):
        ptr = self.current if p is None else p
        ptr.clearContinue()
        if ptr not in self.nextframes:
            return
        for f in self.nextframes[ptr]:
            self.clearContinue(f)

    def clearReturn(self, p=None):
        ptr = self.current if p is None else p
        ptr.clearReturn()
        if ptr not in self.nextframes:
            return
        for f in self.nextframes[ptr]:
            self.clearReturn(f)

    def clearReturnVariable(self, p=None):
        ptr = self.current if p is None else p
        ptr.clearReturnVariable()
        if ptr not in self.nextframes:
            return
        for f in self.nextframes[ptr]:
            self.clearReturnVariable(f)


class ScopeFrame(object):

    def __init__(self, name, ftype=None):
        self.name = name
        self.ftype = ftype
        self.variables = OrderedDict()
        self.nonlocals = []
        self.globals = []
        self.functions = OrderedDict()
        self.unresolved_break = []
        self.unresolved_continue = []
        self.unresolved_return = []
        self.returnvariable = None

    def getNamePrefix(self):
        return str(self.name)

    def addVariable(self, name, var):
        self.variables[name] = var

    def addNonlocal(self, name):
        self.nonlocals.append(name)

    def addGlobal(self, name):
        self.globals.append(name)

    def addFunction(self, func):
        name = func.name
        self.functions[name] = func

    def searchVariable(self, name):
        if name not in self.variables:
            return None
        return self.variables[name]

    def searchNonlocal(self, name):
        if name not in self.nonlocals:
            return None
        return name

    def searchGlobal(self, name):
        if name not in self.globals:
            return None
        return name

    def searchFunction(self, name):
        if name not in self.functions:
            return None
        return self.functions[name]

    # getter to all-inclusive information
    def getVariables(self):
        return tuple(self.variables)

    def getNonlocals(self):
        return tuple(self.nonlocals)

    def getGlobals(self):
        return tuple(self.globals)

    def getFunctions(self):
        return self.functions

    def addBreak(self, count):
        self.unresolved_break.append(count)

    def addContinue(self, count):
        self.unresolved_continue.append(count)

    def addReturn(self, count, value):
        self.unresolved_return.append((count, value))

    def hasBreak(self):
        if self.unresolved_break:
            return True
        return False

    def hasContinue(self):
        if self.unresolved_continue:
            return True
        return False

    def hasReturn(self):
        if self.unresolved_return:
            return True
        return False

    def getBreak(self):
        return self.unresolved_break

    def getContinue(self):
        return self.unresolved_continue

    def getReturn(self):
        return self.unresolved_return

    def clearBreak(self):
        self.unresolved_break = []

    def clearContinue(self):
        self.unresolved_continue = []

    def clearReturn(self):
        self.unresolved_return = []

    def clearReturnVariable(self):
        self.returnvariable = None

    def setReturnVariable(self, var):
        self.returnvariable = var

    def getReturnVariable(self):
        return self.returnvariable

    def __hash__(self):
        return hash((self.name, self.ftype, id(self)))

    def __eq__(self, other):
        if type(self) != type(other):
            return False
        if self.name != other.name:
            return False
        if self.ftype != other.ftype:
            return False
        if id(self) != id(other):
            return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import sort

expected_verilog = """
module simsort #
 (
  parameter WIDTH = 32
 )
 (
 );

  reg CLK;
  reg RST;
  reg [WIDTH-1:0] input_0;
  reg [WIDTH-1:0] input_1;
  reg [WIDTH-1:0] input_2;
  reg [WIDTH-1:0] input_3;
  wire [WIDTH-1:0] output_0;
  wire [WIDTH-1:0] output_1;
  wire [WIDTH-1:0] output_2;
  wire [WIDTH-1:0] output_3;
  reg kick;
  wire busy;

  sort #
   (
    .WIDTH(WIDTH)
   )
  uut
   (
    .CLK(CLK),
    .RST(RST),
    .input_0(input_0),
    .input_1(input_1),
    .input_2(input_2),
    .input_3(input_3),
    .output_0(output_0),
    .output_1(output_1),
    .output_2(output_2),
    .output_3(output_3),
    .kick(kick),
    .busy(busy)
   );

  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end

  initial begin
    CLK = 0; 
    forever begin
      #5 CLK = !CLK;
    end
  end

  initial begin
    RST = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
  end

  initial begin
    input_0 = 100;
    input_1 = 99;
    input_2 = 98;
    input_3 = 97;
    kick = 0;
    wait(RST);
    @(posedge CLK);
    #1;
    wait(!RST);
    @(posedge CLK);
    #1;
    @(posedge CLK);
    #1;
    @(posedge CLK);
    #1;
    kick = 1;
    @(posedge CLK);
    #1;
    kick = 0;
  end

  initial begin
    #100;
    wait(kick);
    @(posedge CLK);
    #1;
    wait(busy);
    @(posedge CLK);
    #1;
    wait(!busy);
    @(posedge CLK);
    #1;
    $finish;
  end

endmodule

module sort #
 (
  parameter WIDTH = 32
 )
 (
  input CLK,
  input RST,
  input [WIDTH-1:0] input_0,
  input [WIDTH-1:0] input_1,
  input [WIDTH-1:0] input_2,
  input [WIDTH-1:0] input_3,
  output [WIDTH-1:0] output_0,
  output [WIDTH-1:0] output_1,
  output [WIDTH-1:0] output_2,
  output [WIDTH-1:0] output_3,
  input kick,
  output reg busy
 );

  reg [WIDTH-1:0] registers_0;
  reg [WIDTH-1:0] registers_1;
  reg [WIDTH-1:0] registers_2;
  reg [WIDTH-1:0] registers_3;

  assign output_0 = registers_0;
  assign output_1 = registers_1;
  assign output_2 = registers_2;
  assign output_3 = registers_3;

  reg [(32 - 1):0] fsm;

  localparam fsm_init = 0;

  wire [(WIDTH - 1):0] small_0;
  wire [(WIDTH - 1):0] large_0;
  assign small_0 = (((registers_0 < registers_1))? registers_0 : registers_1);
  assign large_0 = (((registers_0 < registers_1))? registers_1 : registers_0);

  wire [(WIDTH - 1):0] small_1;
  wire [(WIDTH - 1):0] large_1;
  assign small_1 = (((large_0 < registers_2))? large_0 : registers_2);
  assign large_1 = (((large_0 < registers_2))? registers_2 : large_0);

  wire [(WIDTH - 1):0] small_2;
  wire [(WIDTH - 1):0] large_2;
  assign small_2 = (((large_1 < registers_3))? large_1 : registers_3);
  assign large_2 = (((large_1 < registers_3))? registers_3 : large_1);

  wire [(WIDTH - 1):0] small_3;
  wire [(WIDTH - 1):0] large_3;
  assign small_3 = (((registers_0 < registers_1))? registers_0 : registers_1);
  assign large_3 = (((registers_0 < registers_1))? registers_1 : registers_0);

  wire [(WIDTH - 1):0] small_4;
  wire [(WIDTH - 1):0] large_4;
  assign small_4 = (((large_3 < registers_2))? large_3 : registers_2);
  assign large_4 = (((large_3 < registers_2))? registers_2 : large_3);

  wire [(WIDTH - 1):0] small_5;
  wire [(WIDTH - 1):0] large_5;
  assign small_5 = (((registers_0 < registers_1))? registers_0 : registers_1);
  assign large_5 = (((registers_0 < registers_1))? registers_1 : registers_0);

  localparam fsm_1 = 1;
  localparam fsm_2 = 2;
  localparam fsm_3 = 3;
  localparam fsm_4 = 4;
  localparam fsm_5 = 5;

  always @(posedge CLK) begin
    if(RST) begin
      busy <= 0;
      registers_0 <= 0;
      registers_1 <= 0;
      registers_2 <= 0;
      registers_3 <= 0;
      fsm <= fsm_init;
    end else begin
      $display("registers_0: %d", registers_0);
      $display("registers_1: %d", registers_1);
      $display("registers_2: %d", registers_2);
      $display("registers_3: %d", registers_3);
      case(fsm)
        fsm_init: begin
          if(kick) begin
            registers_0 <= input_0;
            registers_1 <= input_1;
            registers_2 <= input_2;
            registers_3 <= input_3;
          end
          if(kick) begin
            busy <= 1;
          end
          if(kick) begin
            fsm <= fsm_1;
          end 
        end
        fsm_1: begin
          registers_0 <= small_0;
          registers_1 <= small_1;
          registers_2 <= small_2;
          registers_3 <= large_2;
          fsm <= fsm_2;
        end
        fsm_2: begin
          registers_0 <= small_3;
          registers_1 <= small_4;
          registers_2 <= large_4;
          registers_3 <= registers_3;
          fsm <= fsm_3;
        end
        fsm_3: begin
          registers_0 <= small_5;
          registers_1 <= large_5;
          registers_2 <= registers_2;
          registers_3 <= registers_3;
          fsm <= fsm_4;
        end
        fsm_4: begin
          registers_0 <= registers_0;
          registers_1 <= registers_1;
          registers_2 <= registers_2;
          registers_3 <= registers_3;
          fsm <= fsm_5;
        end
        fsm_5: begin
          busy <= 0;
          fsm <= fsm_init;
        end
      endcase
    end
  end
endmodule
"""

expected_rslt = """\
VCD info: dumpfile uut.vcd opened for output.
registers_0:          x
registers_1:          x
registers_2:          x
registers_3:          x
registers_0:          x
registers_1:          x
registers_2:          x
registers_3:          x
registers_0:          x
registers_1:          x
registers_2:          x
registers_3:          x
registers_0:          x
registers_1:          x
registers_2:          x
registers_3:          x
registers_0:          x
registers_1:          x
registers_2:          x
registers_3:          x
registers_0:          x
registers_1:          x
registers_2:          x
registers_3:          x
registers_0:          x
registers_1:          x
registers_2:          x
registers_3:          x
registers_0:          x
registers_1:          x
registers_2:          x
registers_3:          x
registers_0:          x
registers_1:          x
registers_2:          x
registers_3:          x
registers_0:          x
registers_1:          x
registers_2:          x
registers_3:          x
registers_0:          0
registers_1:          0
registers_2:          0
registers_3:          0
registers_0:          0
registers_1:          0
registers_2:          0
registers_3:          0
registers_0:          0
registers_1:          0
registers_2:          0
registers_3:          0
registers_0:          0
registers_1:          0
registers_2:          0
registers_3:          0
registers_0:        100
registers_1:         99
registers_2:         98
registers_3:         97
registers_0:         99
registers_1:         98
registers_2:         97
registers_3:        100
registers_0:         98
registers_1:         97
registers_2:         99
registers_3:        100
registers_0:         97
registers_1:         98
registers_2:         99
registers_3:        100
registers_0:         97
registers_1:         98
registers_2:         99
registers_3:        100
registers_0:         97
registers_1:         98
registers_2:         99
registers_3:        100
"""

def test():
    veriloggen.reset()
    sort_module = sort.mkSimSort()
    sort_code = sort_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == sort_code)



from __future__ import absolute_import
from __future__ import print_function
import sys
import os

# the next line can be removed after installation
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))))

from veriloggen import *
import veriloggen.thread as vthread


def mkLed():
    m = Module('blinkled')
    clk = m.Input('CLK')
    rst = m.Input('RST')
    led = m.OutputReg('LED', 8, initval=0)

    def blink():
        vthread.set_parallel()
        # parallel execution like non-blocking subst
        a = 0
        b = 1
        vthread.unset_parallel()

        for _ in range(10):
            vthread.set_parallel()
            # parallel execution like non-blocking subst
            led.value = a
            # work as 'swap'
            a = b
            b = a
            vthread.unset_parallel()

        vthread.finish()

    th = vthread.Thread(m, 'th_blink', clk, rst, blink)
    fsm = th.start()

    return m


def mkTest():
    m = Module('test')

    # target instance
    led = mkLed()

    # copy paras and ports
    params = m.copy_params(led)
    ports = m.copy_sim_ports(led)

    clk = ports['CLK']
    rst = ports['RST']

    uut = m.Instance(led, 'uut',
                     params=m.connect_params(led),
                     ports=m.connect_ports(led))

    # simulation.setup_waveform(m, uut)
    simulation.setup_clock(m, clk, hperiod=5)
    init = simulation.setup_reset(m, rst, m.make_reset(), period=100)

    init.add(
        Delay(10000),
        Systask('finish'),
    )

    return m


if __name__ == '__main__':
    test = mkTest()
    verilog = test.to_verilog('tmp.v')
    print(verilog)

    sim = simulation.Simulator(test)
    rslt = sim.run()
    print(rslt)



from __future__ import absolute_import
from __future__ import print_function
import veriloggen
import fsm_delayed_cond

expected_verilog = """
module test;

  reg CLK;
  reg RST;
  wire valid;

  blinkled
  uut
  (
    .CLK(CLK),
    .RST(RST),
    .valid(valid)
  );


  initial begin
    $dumpfile("uut.vcd");
    $dumpvars(0, uut);
  end


  initial begin
    CLK = 0;
    forever begin
      #5 CLK = !CLK;
    end
  end


  initial begin
    RST = 0;
    #100;
    RST = 1;
    #100;
    RST = 0;
    #1000;
    $finish;
  end


endmodule



module blinkled
(
  input CLK,
  input RST,
  output reg valid
);

  reg [32-1:0] count;
  reg [32-1:0] fsm;
  localparam fsm_init = 0;
  reg [32-1:0] _d1_fsm;
  reg _fsm_cond_4_0_1;
  reg [32-1:0] _d2_fsm;
  reg _fsm_cond_4_1_1;
  reg _fsm_cond_4_1_2;
  reg [32-1:0] _d3_fsm;
  reg _fsm_cond_4_2_1;
  reg _fsm_cond_4_2_2;
  reg _fsm_cond_4_2_3;
  reg _fsm_cond_13_3_1;
  reg _fsm_cond_13_4_1;
  reg _fsm_cond_13_4_2;
  reg _fsm_cond_13_5_1;
  reg _fsm_cond_13_5_2;
  reg _fsm_cond_13_5_3;
  reg [32-1:0] _d4_fsm;
  reg _fsm_cond_13_6_1;
  reg _fsm_cond_13_6_2;
  reg _fsm_cond_13_6_3;
  reg _fsm_cond_13_6_4;
  reg _fsm_cond_14_7_1;
  reg _fsm_cond_14_8_1;
  reg _fsm_cond_14_8_2;
  reg _fsm_cond_14_9_1;
  reg _fsm_cond_14_9_2;
  reg _fsm_cond_14_9_3;
  reg _fsm_cond_14_10_1;
  reg _fsm_cond_14_10_2;
  reg _fsm_cond_14_10_3;
  reg _fsm_cond_14_10_4;
  reg _fsm_cond_15_11_1;
  reg _fsm_cond_15_12_1;
  reg _fsm_cond_15_12_2;
  reg _fsm_cond_15_13_1;
  reg _fsm_cond_15_13_2;
  reg _fsm_cond_15_13_3;
  reg _fsm_cond_15_14_1;
  reg _fsm_cond_15_14_2;
  reg _fsm_cond_15_14_3;
  reg _fsm_cond_15_14_4;
  reg _fsm_cond_16_15_1;
  reg _fsm_cond_16_16_1;
  reg _fsm_cond_16_16_2;
  reg _fsm_cond_16_17_1;
  reg _fsm_cond_16_17_2;
  reg _fsm_cond_16_17_3;
  reg _fsm_cond_16_18_1;
  reg _fsm_cond_16_18_2;
  reg _fsm_cond_16_18_3;
  reg _fsm_cond_16_18_4;
  reg _fsm_cond_17_19_1;
  reg _fsm_cond_17_20_1;
  reg _fsm_cond_17_20_2;
  reg _fsm_cond_17_21_1;
  reg _fsm_cond_17_21_2;
  reg _fsm_cond_17_21_3;
  reg _fsm_cond_17_22_1;
  reg _fsm_cond_17_22_2;
  reg _fsm_cond_17_22_3;
  reg _fsm_cond_17_22_4;
  reg _fsm_cond_18_23_1;
  reg _fsm_cond_18_24_1;
  reg _fsm_cond_18_24_2;
  reg _fsm_cond_18_25_1;
  reg _fsm_cond_18_25_2;
  reg _fsm_cond_18_25_3;
  reg _fsm_cond_18_26_1;
  reg _fsm_cond_18_26_2;
  reg _fsm_cond_18_26_3;
  reg _fsm_cond_18_26_4;
  reg _fsm_cond_19_27_1;
  reg _fsm_cond_19_28_1;
  reg _fsm_cond_19_28_2;
  reg _fsm_cond_19_29_1;
  reg _fsm_cond_19_29_2;
  reg _fsm_cond_19_29_3;
  reg _fsm_cond_19_30_1;
  reg _fsm_cond_19_30_2;
  reg _fsm_cond_19_30_3;
  reg _fsm_cond_19_30_4;
  reg _fsm_cond_20_31_1;
  reg _fsm_cond_20_32_1;
  reg _fsm_cond_20_32_2;
  reg _fsm_cond_20_33_1;
  reg _fsm_cond_20_33_2;
  reg _fsm_cond_20_33_3;
  reg _fsm_cond_20_34_1;
  reg _fsm_cond_20_34_2;
  reg _fsm_cond_20_34_3;
  reg _fsm_cond_20_34_4;
  localparam fsm_1 = 1;
  localparam fsm_2 = 2;
  localparam fsm_3 = 3;
  localparam fsm_4 = 4;
  localparam fsm_5 = 5;
  localparam fsm_6 = 6;
  localparam fsm_7 = 7;
  localparam fsm_8 = 8;
  localparam fsm_9 = 9;
  localparam fsm_10 = 10;
  localparam fsm_11 = 11;
  localparam fsm_12 = 12;
  localparam fsm_13 = 13;
  localparam fsm_14 = 14;
  localparam fsm_15 = 15;
  localparam fsm_16 = 16;
  localparam fsm_17 = 17;
  localparam fsm_18 = 18;
  localparam fsm_19 = 19;
  localparam fsm_20 = 20;
  localparam fsm_21 = 21;

  always @(posedge CLK) begin
    if(RST) begin
      count <= 0;
      fsm <= fsm_init;
      _d1_fsm <= fsm_init;
      _d2_fsm <= fsm_init;
      _d3_fsm <= fsm_init;
      _d4_fsm <= fsm_init;
      valid <= 0;
      _fsm_cond_4_0_1 <= 0;
      _fsm_cond_4_1_1 <= 0;
      _fsm_cond_4_1_2 <= 0;
      _fsm_cond_4_2_1 <= 0;
      _fsm_cond_4_2_2 <= 0;
      _fsm_cond_4_2_3 <= 0;
      _fsm_cond_13_3_1 <= 0;
      _fsm_cond_13_4_1 <= 0;
      _fsm_cond_13_4_2 <= 0;
      _fsm_cond_13_5_1 <= 0;
      _fsm_cond_13_5_2 <= 0;
      _fsm_cond_13_5_3 <= 0;
      _fsm_cond_13_6_1 <= 0;
      _fsm_cond_13_6_2 <= 0;
      _fsm_cond_13_6_3 <= 0;
      _fsm_cond_13_6_4 <= 0;
      _fsm_cond_14_7_1 <= 0;
      _fsm_cond_14_8_1 <= 0;
      _fsm_cond_14_8_2 <= 0;
      _fsm_cond_14_9_1 <= 0;
      _fsm_cond_14_9_2 <= 0;
      _fsm_cond_14_9_3 <= 0;
      _fsm_cond_14_10_1 <= 0;
      _fsm_cond_14_10_2 <= 0;
      _fsm_cond_14_10_3 <= 0;
      _fsm_cond_14_10_4 <= 0;
      _fsm_cond_15_11_1 <= 0;
      _fsm_cond_15_12_1 <= 0;
      _fsm_cond_15_12_2 <= 0;
      _fsm_cond_15_13_1 <= 0;
      _fsm_cond_15_13_2 <= 0;
      _fsm_cond_15_13_3 <= 0;
      _fsm_cond_15_14_1 <= 0;
      _fsm_cond_15_14_2 <= 0;
      _fsm_cond_15_14_3 <= 0;
      _fsm_cond_15_14_4 <= 0;
      _fsm_cond_16_15_1 <= 0;
      _fsm_cond_16_16_1 <= 0;
      _fsm_cond_16_16_2 <= 0;
      _fsm_cond_16_17_1 <= 0;
      _fsm_cond_16_17_2 <= 0;
      _fsm_cond_16_17_3 <= 0;
      _fsm_cond_16_18_1 <= 0;
      _fsm_cond_16_18_2 <= 0;
      _fsm_cond_16_18_3 <= 0;
      _fsm_cond_16_18_4 <= 0;
      _fsm_cond_17_19_1 <= 0;
      _fsm_cond_17_20_1 <= 0;
      _fsm_cond_17_20_2 <= 0;
      _fsm_cond_17_21_1 <= 0;
      _fsm_cond_17_21_2 <= 0;
      _fsm_cond_17_21_3 <= 0;
      _fsm_cond_17_22_1 <= 0;
      _fsm_cond_17_22_2 <= 0;
      _fsm_cond_17_22_3 <= 0;
      _fsm_cond_17_22_4 <= 0;
      _fsm_cond_18_23_1 <= 0;
      _fsm_cond_18_24_1 <= 0;
      _fsm_cond_18_24_2 <= 0;
      _fsm_cond_18_25_1 <= 0;
      _fsm_cond_18_25_2 <= 0;
      _fsm_cond_18_25_3 <= 0;
      _fsm_cond_18_26_1 <= 0;
      _fsm_cond_18_26_2 <= 0;
      _fsm_cond_18_26_3 <= 0;
      _fsm_cond_18_26_4 <= 0;
      _fsm_cond_19_27_1 <= 0;
      _fsm_cond_19_28_1 <= 0;
      _fsm_cond_19_28_2 <= 0;
      _fsm_cond_19_29_1 <= 0;
      _fsm_cond_19_29_2 <= 0;
      _fsm_cond_19_29_3 <= 0;
      _fsm_cond_19_30_1 <= 0;
      _fsm_cond_19_30_2 <= 0;
      _fsm_cond_19_30_3 <= 0;
      _fsm_cond_19_30_4 <= 0;
      _fsm_cond_20_31_1 <= 0;
      _fsm_cond_20_32_1 <= 0;
      _fsm_cond_20_32_2 <= 0;
      _fsm_cond_20_33_1 <= 0;
      _fsm_cond_20_33_2 <= 0;
      _fsm_cond_20_33_3 <= 0;
      _fsm_cond_20_34_1 <= 0;
      _fsm_cond_20_34_2 <= 0;
      _fsm_cond_20_34_3 <= 0;
      _fsm_cond_20_34_4 <= 0;
    end else begin
      count <= count + 1;
      _d1_fsm <= fsm;
      _d2_fsm <= _d1_fsm;
      _d3_fsm <= _d2_fsm;
      _d4_fsm <= _d3_fsm;
      case(_d4_fsm)
        fsm_13: begin
          if(_fsm_cond_13_6_4) begin
            valid <= 0;
          end 
        end
        fsm_14: begin
          if(_fsm_cond_14_10_4) begin
            valid <= 0;
          end 
        end
        fsm_15: begin
          if(_fsm_cond_15_14_4) begin
            valid <= 0;
          end 
        end
        fsm_16: begin
          if(_fsm_cond_16_18_4) begin
            valid <= 0;
          end 
        end
        fsm_17: begin
          if(_fsm_cond_17_22_4) begin
            valid <= 0;
          end 
        end
        fsm_18: begin
          if(_fsm_cond_18_26_4) begin
            valid <= 0;
          end 
        end
        fsm_19: begin
          if(_fsm_cond_19_30_4) begin
            valid <= 0;
          end 
        end
        fsm_20: begin
          if(_fsm_cond_20_34_4) begin
            valid <= 0;
          end 
        end
      endcase
      case(_d3_fsm)
        fsm_4: begin
          if(_fsm_cond_4_2_3) begin
            valid <= 0;
          end 
        end
        fsm_13: begin
          if(_fsm_cond_13_5_3) begin
            valid <= 1;
          end 
          _fsm_cond_13_6_4 <= _fsm_cond_13_6_3;
        end
        fsm_14: begin
          if(_fsm_cond_14_9_3) begin
            valid <= 1;
          end 
          _fsm_cond_14_10_4 <= _fsm_cond_14_10_3;
        end
        fsm_15: begin
          if(_fsm_cond_15_13_3) begin
            valid <= 1;
          end 
          _fsm_cond_15_14_4 <= _fsm_cond_15_14_3;
        end
        fsm_16: begin
          if(_fsm_cond_16_17_3) begin
            valid <= 1;
          end 
          _fsm_cond_16_18_4 <= _fsm_cond_16_18_3;
        end
        fsm_17: begin
          if(_fsm_cond_17_21_3) begin
            valid <= 1;
          end 
          _fsm_cond_17_22_4 <= _fsm_cond_17_22_3;
        end
        fsm_18: begin
          if(_fsm_cond_18_25_3) begin
            valid <= 1;
          end 
          _fsm_cond_18_26_4 <= _fsm_cond_18_26_3;
        end
        fsm_19: begin
          if(_fsm_cond_19_29_3) begin
            valid <= 1;
          end 
          _fsm_cond_19_30_4 <= _fsm_cond_19_30_3;
        end
        fsm_20: begin
          if(_fsm_cond_20_33_3) begin
            valid <= 1;
          end 
          _fsm_cond_20_34_4 <= _fsm_cond_20_34_3;
        end
      endcase
      case(_d2_fsm)
        fsm_4: begin
          if(_fsm_cond_4_1_2) begin
            valid <= 1;
          end 
          _fsm_cond_4_2_3 <= _fsm_cond_4_2_2;
        end
        fsm_13: begin
          if(_fsm_cond_13_4_2) begin
            valid <= 1;
          end 
          _fsm_cond_13_5_3 <= _fsm_cond_13_5_2;
          _fsm_cond_13_6_3 <= _fsm_cond_13_6_2;
        end
        fsm_14: begin
          if(_fsm_cond_14_8_2) begin
            valid <= 1;
          end 
          _fsm_cond_14_9_3 <= _fsm_cond_14_9_2;
          _fsm_cond_14_10_3 <= _fsm_cond_14_10_2;
        end
        fsm_15: begin
          if(_fsm_cond_15_12_2) begin
            valid <= 1;
          end 
          _fsm_cond_15_13_3 <= _fsm_cond_15_13_2;
          _fsm_cond_15_14_3 <= _fsm_cond_15_14_2;
        end
        fsm_16: begin
          if(_fsm_cond_16_16_2) begin
            valid <= 1;
          end 
          _fsm_cond_16_17_3 <= _fsm_cond_16_17_2;
          _fsm_cond_16_18_3 <= _fsm_cond_16_18_2;
        end
        fsm_17: begin
          if(_fsm_cond_17_20_2) begin
            valid <= 1;
          end 
          _fsm_cond_17_21_3 <= _fsm_cond_17_21_2;
          _fsm_cond_17_22_3 <= _fsm_cond_17_22_2;
        end
        fsm_18: begin
          if(_fsm_cond_18_24_2) begin
            valid <= 1;
          end 
          _fsm_cond_18_25_3 <= _fsm_cond_18_25_2;
          _fsm_cond_18_26_3 <= _fsm_cond_18_26_2;
        end
        fsm_19: begin
          if(_fsm_cond_19_28_2) begin
            valid <= 1;
          end 
          _fsm_cond_19_29_3 <= _fsm_cond_19_29_2;
          _fsm_cond_19_30_3 <= _fsm_cond_19_30_2;
        end
        fsm_20: begin
          if(_fsm_cond_20_32_2) begin
            valid <= 1;
          end 
          _fsm_cond_20_33_3 <= _fsm_cond_20_33_2;
          _fsm_cond_20_34_3 <= _fsm_cond_20_34_2;
        end
      endcase
      case(_d1_fsm)
        fsm_4: begin
          if(_fsm_cond_4_0_1) begin
            valid <= 1;
          end 
          _fsm_cond_4_1_2 <= _fsm_cond_4_1_1;
          _fsm_cond_4_2_2 <= _fsm_cond_4_2_1;
        end
        fsm_13: begin
          if(_fsm_cond_13_3_1) begin
            valid <= 1;
          end 
          _fsm_cond_13_4_2 <= _fsm_cond_13_4_1;
          _fsm_cond_13_5_2 <= _fsm_cond_13_5_1;
          _fsm_cond_13_6_2 <= _fsm_cond_13_6_1;
        end
        fsm_14: begin
          if(_fsm_cond_14_7_1) begin
            valid <= 1;
          end 
          _fsm_cond_14_8_2 <= _fsm_cond_14_8_1;
          _fsm_cond_14_9_2 <= _fsm_cond_14_9_1;
          _fsm_cond_14_10_2 <= _fsm_cond_14_10_1;
        end
        fsm_15: begin
          if(_fsm_cond_15_11_1) begin
            valid <= 1;
          end 
          _fsm_cond_15_12_2 <= _fsm_cond_15_12_1;
          _fsm_cond_15_13_2 <= _fsm_cond_15_13_1;
          _fsm_cond_15_14_2 <= _fsm_cond_15_14_1;
        end
        fsm_16: begin
          if(_fsm_cond_16_15_1) begin
            valid <= 1;
          end 
          _fsm_cond_16_16_2 <= _fsm_cond_16_16_1;
          _fsm_cond_16_17_2 <= _fsm_cond_16_17_1;
          _fsm_cond_16_18_2 <= _fsm_cond_16_18_1;
        end
        fsm_17: begin
          if(_fsm_cond_17_19_1) begin
            valid <= 1;
          end 
          _fsm_cond_17_20_2 <= _fsm_cond_17_20_1;
          _fsm_cond_17_21_2 <= _fsm_cond_17_21_1;
          _fsm_cond_17_22_2 <= _fsm_cond_17_22_1;
        end
        fsm_18: begin
          if(_fsm_cond_18_23_1) begin
            valid <= 1;
          end 
          _fsm_cond_18_24_2 <= _fsm_cond_18_24_1;
          _fsm_cond_18_25_2 <= _fsm_cond_18_25_1;
          _fsm_cond_18_26_2 <= _fsm_cond_18_26_1;
        end
        fsm_19: begin
          if(_fsm_cond_19_27_1) begin
            valid <= 1;
          end 
          _fsm_cond_19_28_2 <= _fsm_cond_19_28_1;
          _fsm_cond_19_29_2 <= _fsm_cond_19_29_1;
          _fsm_cond_19_30_2 <= _fsm_cond_19_30_1;
        end
        fsm_20: begin
          if(_fsm_cond_20_31_1) begin
            valid <= 1;
          end 
          _fsm_cond_20_32_2 <= _fsm_cond_20_32_1;
          _fsm_cond_20_33_2 <= _fsm_cond_20_33_1;
          _fsm_cond_20_34_2 <= _fsm_cond_20_34_1;
        end
      endcase
      case(fsm)
        fsm_init: begin
          fsm <= fsm_1;
        end
        fsm_1: begin
          fsm <= fsm_2;
        end
        fsm_2: begin
          fsm <= fsm_3;
        end
        fsm_3: begin
          fsm <= fsm_4;
        end
        fsm_4: begin
          if(count >= 16) begin
            valid <= 1;
          end 
          _fsm_cond_4_0_1 <= count >= 16;
          _fsm_cond_4_1_1 <= count >= 16;
          _fsm_cond_4_2_1 <= count >= 16;
          if(count >= 16) begin
            fsm <= fsm_5;
          end 
        end
        fsm_5: begin
          fsm <= fsm_6;
        end
        fsm_6: begin
          fsm <= fsm_7;
        end
        fsm_7: begin
          fsm <= fsm_8;
        end
        fsm_8: begin
          fsm <= fsm_9;
        end
        fsm_9: begin
          fsm <= fsm_10;
        end
        fsm_10: begin
          fsm <= fsm_11;
        end
        fsm_11: begin
          fsm <= fsm_12;
        end
        fsm_12: begin
          fsm <= fsm_13;
        end
        fsm_13: begin
          _fsm_cond_13_3_1 <= count >= 32;
          _fsm_cond_13_4_1 <= count >= 32;
          _fsm_cond_13_5_1 <= count >= 32;
          _fsm_cond_13_6_1 <= count >= 32;
          if(count >= 32) begin
            fsm <= fsm_14;
          end 
        end
        fsm_14: begin
          _fsm_cond_14_7_1 <= count >= 32;
          _fsm_cond_14_8_1 <= count >= 32;
          _fsm_cond_14_9_1 <= count >= 32;
          _fsm_cond_14_10_1 <= count >= 32;
          if(count >= 32) begin
            fsm <= fsm_15;
          end 
        end
        fsm_15: begin
          _fsm_cond_15_11_1 <= count >= 32;
          _fsm_cond_15_12_1 <= count >= 32;
          _fsm_cond_15_13_1 <= count >= 32;
          _fsm_cond_15_14_1 <= count >= 32;
          if(count >= 32) begin
            fsm <= fsm_16;
          end 
        end
        fsm_16: begin
          _fsm_cond_16_15_1 <= count >= 32;
          _fsm_cond_16_16_1 <= count >= 32;
          _fsm_cond_16_17_1 <= count >= 32;
          _fsm_cond_16_18_1 <= count >= 32;
          if(count >= 32) begin
            fsm <= fsm_17;
          end 
        end
        fsm_17: begin
          _fsm_cond_17_19_1 <= count >= 32;
          _fsm_cond_17_20_1 <= count >= 32;
          _fsm_cond_17_21_1 <= count >= 32;
          _fsm_cond_17_22_1 <= count >= 32;
          if(count >= 32) begin
            fsm <= fsm_18;
          end 
        end
        fsm_18: begin
          _fsm_cond_18_23_1 <= count >= 32;
          _fsm_cond_18_24_1 <= count >= 32;
          _fsm_cond_18_25_1 <= count >= 32;
          _fsm_cond_18_26_1 <= count >= 32;
          if(count >= 32) begin
            fsm <= fsm_19;
          end 
        end
        fsm_19: begin
          _fsm_cond_19_27_1 <= count >= 32;
          _fsm_cond_19_28_1 <= count >= 32;
          _fsm_cond_19_29_1 <= count >= 32;
          _fsm_cond_19_30_1 <= count >= 32;
          if(count >= 32) begin
            fsm <= fsm_20;
          end 
        end
        fsm_20: begin
          _fsm_cond_20_31_1 <= count >= 32;
          _fsm_cond_20_32_1 <= count >= 32;
          _fsm_cond_20_33_1 <= count >= 32;
          _fsm_cond_20_34_1 <= count >= 32;
          if(count >= 32) begin
            fsm <= fsm_21;
          end 
        end
      endcase
    end
  end


endmodule
"""

def test():
    veriloggen.reset()
    test_module = fsm_delayed_cond.mkTest()
    code = test_module.to_verilog()

    from pyverilog.vparser.parser import VerilogParser
    from pyverilog.ast_code_generator.codegen import ASTCodeGenerator
    parser = VerilogParser()
    expected_ast = parser.parse(expected_verilog)
    codegen = ASTCodeGenerator()
    expected_code = codegen.visit(expected_ast)

    assert(expected_code == code)
